@@ -2,7 +2,7 @@
 //!
 //! This library will parse gemtext into various [Nodes](GemtextNode)
 
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq, Clone)]
 /// A singular gemtext node.
 pub enum GemtextNode {
     /// A pure text block. The string contained within is the entire line of text
@@ -51,8 +51,8 @@ pub enum GemtextNode {
     EmptyLine,
 }
 
-impl std::fmt::Display for GemtextNode {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+impl core::fmt::Display for GemtextNode {
+    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
         match self {
             GemtextNode::EmptyLine => write!(f, ""),
             GemtextNode::Text(s) => write!(f, "{}", s),
