@@ -23,7 +23,9 @@
 /// - An authority (example.com:1234),
 /// - A path (hello/world.gmi)
 /// - A query (?user=23)
-#[derive(Debug)]
+///
+/// An easy way to construct the Url struct is using `try_from()`
+#[derive(Debug, Clone)]
 pub struct Url {
     /// The scheme of the URL
     pub scheme: Option<String>,
@@ -35,8 +37,8 @@ pub struct Url {
     pub query: Option<Query>,
 }
 
-impl std::fmt::Display for Url {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+impl core::fmt::Display for Url {
+    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
         // Print the scheme
         match &self.scheme {
             Some(s) => write!(f, "{}://", s)?,
@@ -58,7 +60,7 @@ impl std::fmt::Display for Url {
     }
 }
 
-#[derive(Debug, PartialEq, Eq)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
 /// An error during the parsing of a URL
 pub enum UrlParseError {
     /// An error occured during the parsing of the authority
@@ -68,7 +70,26 @@ pub enum UrlParseError {
     EmptyURL,
 }
 
-impl Url {
+impl core::fmt::Display for UrlParseError {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        match self {
+            UrlParseError::AuthorityParseError(e) => write!(f, "URL parse error: {}", e),
+            UrlParseError::EmptyURL => write!(f, "URL Parse error: Empty URL"),
+        }
+    }
+}
+
+impl std::error::Error for UrlParseError {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+        match self {
+            UrlParseError::AuthorityParseError(e) => Some(e),
+            UrlParseError::EmptyURL => None,
+        }
+    }
+}
+
+impl core::convert::TryFrom<&str> for Url {
+    type Error = UrlParseError;
     /// Parse a URL from a [`&str`] object.
     ///
     /// # Example:
@@ -76,7 +97,8 @@ impl Url {
     /// # use gmi::url::Url;
     /// # use gmi::url::UrlParseError;
     /// # fn main() -> Result<(), UrlParseError> {
-    /// let url = Url::from_str("gemini://example.com")?;
+    /// use std::convert::TryFrom;
+    /// let url = Url::try_from("gemini://example.com")?;
     /// assert_eq!(url.scheme, Some(String::from("gemini")));
     /// assert_eq!(url.authority.host, "example.com");
     /// assert_eq!(url.path, None);
@@ -84,7 +106,7 @@ impl Url {
     /// # Ok(())
     /// # }
     /// ```
-    pub fn from_str(raw_url: &str) -> Result<Self, UrlParseError> {
+    fn try_from(raw_url: &str) -> Result<Self, UrlParseError> {
         if raw_url.trim().is_empty() {
             return Err(UrlParseError::EmptyURL);
         }
@@ -106,7 +128,7 @@ impl Url {
         };
 
         // Now we get the authority
-        let authority = match Authority::parse_str(auth) {
+        let authority = match Authority::try_from(auth) {
             Ok(a) => a,
             Err(e) => return Err(UrlParseError::AuthorityParseError(e)),
         };
@@ -155,7 +177,22 @@ impl Url {
 ///
 /// For more info see [section 3.2 of the
 /// RFC](https://datatracker.ietf.org/doc/html/rfc3986#section-3.2)
-#[derive(Debug, Eq, PartialEq)]
+///
+/// # Constructing the struct
+/// An easy way to construct this struct is to use `try_from()`
+///
+/// ## Example
+/// ```
+/// # use gmi::url::Authority;
+/// # fn main() -> Result<(), gmi::url::AuthorityParseError> {
+/// use std::convert::TryFrom;
+/// let auth = Authority::try_from("example.com:1963")?;
+/// assert_eq!(auth.port, Some(1963));
+/// assert_eq!(auth.host, "example.com");
+/// # Ok(())
+/// # }
+/// ```
+#[derive(Debug, Eq, PartialEq, Clone)]
 pub struct Authority {
     /// The host portion of the authority
     pub host: String,
@@ -163,8 +200,8 @@ pub struct Authority {
     pub port: Option<u16>,
 }
 
-impl std::fmt::Display for Authority {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+impl core::fmt::Display for Authority {
+    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
         write!(
             f,
             "{}{}",
@@ -177,7 +214,7 @@ impl std::fmt::Display for Authority {
     }
 }
 
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq, Clone, Copy)]
 /// A parsing error for the Authority
 ///
 /// This enum contains various possible errors that can occur while
@@ -190,24 +227,20 @@ pub enum AuthorityParseError {
     MissingHost,
 }
 
-impl Authority {
-    /// Parse a str into an Authority.
-    ///
-    /// # Returns:
-    /// A complete [`Authority structure`](Authority)
-    ///
-    /// # Error:
-    /// See [`AuthorityParseError`])
-    /// # Example
-    /// ```
-    /// # use gmi::url::Authority;
-    /// # fn main() -> Result<(), gmi::url::AuthorityParseError> {
-    /// let auth = Authority::parse_str("example.com:1963")?;
-    /// assert_eq!(auth.port, Some(1963));
-    /// assert_eq!(auth.host, "example.com");
-    /// # Ok(())
-    /// # }
-    pub fn parse_str(s: &str) -> Result<Self, AuthorityParseError> {
+impl core::fmt::Display for AuthorityParseError {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        match self {
+            AuthorityParseError::InvalidPort => write!(f, "Error parsing authority: Invalid port"),
+            AuthorityParseError::MissingHost => write!(f, "Error parsing authority: Missing host"),
+        }
+    }
+}
+
+impl std::error::Error for AuthorityParseError {}
+
+impl core::convert::TryFrom<&str> for Authority {
+    type Error = AuthorityParseError;
+    fn try_from(s: &str) -> Result<Self, AuthorityParseError> {
         let host;
         let mut port = None;
         // Split once by colon
@@ -239,40 +272,41 @@ impl Authority {
 /// This part is optional in a URL and specifies the specific resource to access
 ///
 /// This implementation is based on the API of [`std::path::Path`]
+///
+/// # Constructing this struct
+/// You can just use the `from()` implementations for this. You can also just
+/// construct it from its raw parts, although I do not recommend it.
+///
+/// ## Example:
+///```
+/// # use gmi::url::Path;
+/// # fn main() {
+/// let path = Path::from("/help/me");
+/// assert_eq!(path.to_string(), "/help/me");
+/// # }
+/// ```
+
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub struct Path {
     /// The raw path string of the URL
     pub raw_path: String,
 }
 
-impl std::fmt::Display for Path {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl core::fmt::Display for Path {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         write!(f, "{}", self.raw_path)
     }
 }
 
 impl From<&str> for Path {
     fn from(s: &str) -> Self {
-        Self::parse_str(s)
+        Self {
+            raw_path: s.to_owned(),
+        }
     }
 }
 
 impl Path {
-    /// Turns a [`&str`] to a [`Path`]
-    ///
-    /// # Example:
-    /// ```
-    /// # use gmi::url::Path;
-    /// # fn main() {
-    /// let path = Path::parse_str("/help/me");
-    /// assert_eq!(path.to_string(), "/help/me");
-    /// # }
-    pub fn parse_str(raw_string: &str) -> Self {
-        Self {
-            raw_path: raw_string.to_owned(),
-        }
-    }
-
     /// Returns a path with the parent part
     ///
     /// # Example
@@ -316,7 +350,7 @@ impl Path {
     /// ```
     /// # use gmi::url::Path;
     /// # fn main() {
-    /// let path = Path::parse_str("/a/good/path");
+    /// let path = Path::from("/a/good/path");
     /// assert!(path.is_absolute());
     /// # }
     /// ```
@@ -331,7 +365,7 @@ impl Path {
     /// ```
     /// # use gmi::url::Path;
     /// # fn main() {
-    /// let path = Path::parse_str("a/relative/path");
+    /// let path = Path::from("a/relative/path");
     /// assert!(path.is_relative());
     /// # }
     /// ```
@@ -416,7 +450,7 @@ impl Path {
     /// ```
     /// # use gmi::url::Path;
     /// # fn main() {
-    /// let mut path = Path::parse_str("/a/dotted/../path/./with/stuff");
+    /// let mut path = Path::from("/a/dotted/../path/./with/stuff");
     /// path.dedotify();
     /// assert_eq!(path.to_string(), "/a/path/with/stuff");
     /// # }
@@ -495,14 +529,14 @@ impl Path {
 /// This part is optional in a URL and consists of "fragments"
 /// The first query part is the first fragment, and each fragment following is separated by the
 /// character '#'
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq, Clone)]
 pub struct Query {
     /// The various fragments of the query portion of the URL
     pub fragments: Vec<String>,
 }
 
-impl std::fmt::Display for Query {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+impl core::fmt::Display for Query {
+    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
         write!(f, "?{}", self.fragments[0])?;
         for p in self.fragments[1..].iter() {
             write!(f, "#{}", p)?;
@@ -522,7 +556,7 @@ impl From<&str> for Query {
 }
 
 impl Query {
-    /// Parses a query part into separate parts.
+    /// Parses a query part into separate parts. You can also use [`Query::from()`]
     ///
     /// This requires the query part to be already separated from the URL. The str will not start
     /// with a '?', and if it does, the first fragment will contain it
@@ -630,11 +664,12 @@ mod test {
         }
     }
     mod authority {
+        use std::convert::TryFrom;
         use crate::url::*;
         #[test]
         fn authority_parse_str_simple() {
             assert_eq!(
-                Authority::parse_str("example.com"),
+                Authority::try_from("example.com"),
                 Ok(Authority {
                     host: "example.com".to_string(),
                     port: None,
@@ -644,7 +679,7 @@ mod test {
         #[test]
         fn authority_parse_str_with_port() {
             assert_eq!(
-                Authority::parse_str("example.com:1234"),
+                Authority::try_from("example.com:1234"),
                 Ok(Authority {
                     host: "example.com".to_string(),
                     port: Some(1234),
@@ -654,18 +689,18 @@ mod test {
         #[test]
         fn authority_parse_str_invalid_port() {
             assert_eq!(
-                Authority::parse_str("example.com:fjdklg"),
+                Authority::try_from("example.com:fjdklg"),
                 Err(AuthorityParseError::InvalidPort)
             );
         }
         #[test]
         fn authority_parse_str_missing_host() {
             assert_eq!(
-                Authority::parse_str(""),
+                Authority::try_from(""),
                 Err(AuthorityParseError::MissingHost)
             );
             assert_eq!(
-                Authority::parse_str(":1323"),
+                Authority::try_from(":1323"),
                 Err(AuthorityParseError::MissingHost)
             );
         }
